@@ -12,13 +12,19 @@ from .shell import PyreneCmd
 
 
 def main():
+    dot_pyrene = os.path.expanduser('~/.pyrene')
+    dot_pypirc = os.path.expanduser('~/.pypirc')
+
     tempdir = tempfile.mkdtemp(suffix='.pyrene')
-    cmd = PyreneCmd(
-        Network(os.path.expanduser('~/.pyrene')),
-        Directory(tempdir),
-    )
-    line = ' '.join(sys.argv[1:])
+    network = Network(dot_pyrene)
     try:
+        if not os.path.exists(dot_pyrene):
+            network.add_known_repos(dot_pypirc)
+
+        cmd = PyreneCmd(network, Directory(tempdir), dot_pypirc)
+
+        line = ' '.join(sys.argv[1:])
+
         if line:
             cmd.onecmd(line)
         else:
